// 
// compute volume of sphere
// Simple example of Monte Carlo simulation method
// Analytical solution: V = 4/3 pi r^3
//

#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <cmath>
#include <ctime>
using namespace std;

const long NEXP = 10000000;	// max number of experiments
const double R = 2;

// generate random number 0..1
inline double Random()
{
    return (double) rand() / RAND_MAX;
}

// compute volume
double mc_sphere_volume(const double r, long nexp) {
    long S = 0;
    for (long i = 0; i < nexp; i++) {
        // random point in one of 8 quadrants
	double x = r*Random();
	double y = r*Random();
	double z = r*Random();
	if ( x*x + y*y + z*z < r*r ) // is in sphere ?
	    S++;
    }
    double probability = (double) S / nexp;
    double result = 8 * r*r*r * probability;
    return result;
}

int main()
{
    srand(time(0)); // each run is different
    cout << endl;
    cout << "Sphere volume computed by Monte Carlo method" << endl;
    cout << "r = " << R << "  Exact solution: " << 4.0/3.0*M_PI*R*R*R << endl;
    cout << endl;
    for (long n = 1000; n <= NEXP; n*=10) {
        clock_t start = clock();
        cout << "volume = " << mc_sphere_volume(R,n) << "  \t";
        cout << "(" << n << " experiments) " ;
        cout << " Time: ";
        cout << double(clock()-start)/CLOCKS_PER_SEC;
        cout << " s" << endl;
    }
    cout << endl;
}

